summaryrefslogtreecommitdiff
path: root/src/mongo/shell/shell_utils.cpp
diff options
context:
space:
mode:
authorLucas de Castro Borges <lucas@gnuabordo.com.br>2025-02-14 14:26:38 -0300
committerLucas de Castro Borges <lucas@gnuabordo.com.br>2025-02-14 14:26:38 -0300
commit294bc6ecabf14c09c9bc8644704921dcf97cb44e (patch)
tree279b1e0bab53901a1647ac63c1c724f0f789a663 /src/mongo/shell/shell_utils.cpp
parent70be7c27a251621187a1de533462ae2bb1e3bd39 (diff)
parent1e917fd798aa25b7066d4b414b51184f13d5a092 (diff)
Update upstream source from tag 'upstream/6.0.10'debian/6.0.10-1
Update to upstream version '6.0.10' with Debian dir 2d176fa254eee97b139f712fec5709641335a8c3
Diffstat (limited to 'src/mongo/shell/shell_utils.cpp')
-rw-r--r--src/mongo/shell/shell_utils.cpp22
1 files changed, 15 insertions, 7 deletions
diff --git a/src/mongo/shell/shell_utils.cpp b/src/mongo/shell/shell_utils.cpp
index cb82f223f9b..abdbb179bfa 100644
--- a/src/mongo/shell/shell_utils.cpp
+++ b/src/mongo/shell/shell_utils.cpp
@@ -521,14 +521,22 @@ BSONObj numberDecimalsAlmostEqual(const BSONObj& input, void*) {
auto ten = Decimal128(10);
auto exponent = a.toAbs().logarithm(ten).round();
- // Return early if arguments are not the same order of magnitude.
- if (exponent != b.toAbs().logarithm(ten).round()) {
- return BSON("" << false);
- }
+ // Early exit for zero, infinity and NaN cases.
+ if ((a.isZero() && b.isZero()) || (a.isNaN() && b.isNaN()) ||
+ (a.isInfinite() && b.isInfinite() && (a.isNegative() == b.isNegative()))) {
+ return BSON("" << true /* isErrorAcceptable */);
+ } else if (!a.isZero() && !b.isZero()) {
+ // Return early if arguments are not the same order of magnitude.
+ if (exponent != b.toAbs().logarithm(ten).round()) {
+ return BSON("" << false);
+ }
- // Put the whole number behind the decimal point.
- a = a.divide(ten.power(exponent));
- b = b.divide(ten.power(exponent));
+ // Put the whole number behind the decimal point.
+ if (!exponent.isZero()) {
+ a = a.divide(ten.power(exponent));
+ b = b.divide(ten.power(exponent));
+ }
+ }
auto places = third.numberDecimal();
auto isErrorAcceptable = a.subtract(b)